<SCRIPT language="JScript">
/* 
 THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 EITHER EXPRESSED OR IMPLIED,  INCLUDING BUT NOT LIMITED TO THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

 This code is free for both personal and commercial use,  but you are
 expressly forbidden from selling.

 **************************************************************************

 FILE NAME:
   smtpenvl.js

 DESCRIPTION: 
   This is an SMTP transport event sink script that inserts the actual
   (SMTP) sender, recipient and client IP address to the mail header.

 COPYRIGHT:
   Copyright (c) Vamsoft Ltd. 2003, Peter Karsai (peter.karsai@vamsoft.com).
   All rights reserved.

 NOTES:
   The original version of this script can be found at:
     http://www.vamsoft.com

 **************************************************************************
*/

  // declare globals
  var cdoRunNextSink = 0;
 
  /*
    Description:
    Extracts the sender email address from the strAddr parameter. Wide variety of formats
    are accepted, e.g. "Somebody" <user1@somewhere.net>
    
    Parameters:
    strAddr: The formatted address to extract the email address from.
    
    Return Value: The extracted email address.
  */
  function GetSender(strAddr) {
    // declare variables  
    var intIdx = strAddr.indexOf(">");
    var strResult = "";
    
    // cut off name part if present
    if(intIdx >= 0) {
      strResult = strAddr.substring(0, intIdx);
      strResult = strResult.substring(strResult.indexOf('<') + 1, strResult.length);
    }
    else {
      strResult = strAddr;
    }
    
    // return the address trim'med
    return(strResult.replace(/^\s+/g, "").replace(/\s+$/g, ""));
  }
  
  
  /*
    Description:
    Returns the recipient email address list as an array, parsed from the recipient
    address list parameter string.
    The expected format is SMTP:user1@somewhere.net;SMTP:user2@somewhere.net;
    Note that the terminating semicolon must be present.
    
    Parameters:
    strRecipients: The recipient address list.
    
    Return Value: Array of recipient addresses.
  */
  function GetRecipients(strRecipients) {
    // declare local variables
    var arrResult = new Array();
    var intIdx = 1;
    var intStartIdx = 0;
    var strTemp = "";
    
    // get addresses from the semicolon-separated list (terminating semicolon is expected!)
    while(intIdx > 0) {
      intIdx = strRecipients.indexOf(';', intStartIdx);
      if(intIdx >= 0) {
        strTemp = strRecipients.substring(intStartIdx, intIdx);
        arrResult[arrResult.length] = strTemp.substring(strTemp.indexOf(':') + 1, 
          strTemp.length).replace(/^\s+/g, "").replace(/\s+$/g, "");
        intStartIdx = intIdx + 1;
      }
    }
    
    // return with the address list
    return(arrResult);
  }  
  
  /* SCRIPT ENTRY POINT ================================================================================= */
  
  function ISMTPOnArrival::OnArrival(Msg, Status) {
    // declare variables
    var strSender, strRecipients, strIP;
    var arrRecipients = new Array();
    var strRecipientHdrs = "";
    
    // get values
    strSender = new String(Msg.EnvelopeFields.Item("http://schemas.microsoft.com/cdo/smtpenvelope/senderemailaddress").Value);
    strRecipients = new String(Msg.EnvelopeFields.Item("http://schemas.microsoft.com/cdo/smtpenvelope/recipientlist").Value);
    strIP = new String(Msg.EnvelopeFields.Item("http://schemas.microsoft.com/cdo/smtpenvelope/clientipaddress").Value);   
    arrRecipients = GetRecipients(strRecipients);
    for(var x = 0; x < arrRecipients.length; x++) {
      strRecipientHdrs += arrRecipients[x] + ", ";
    }
    if(strRecipientHdrs.length > 0) {
      strRecipientHdrs = strRecipientHdrs.substring(0, strRecipientHdrs.length - 2);
    }
    
    // add SMTP data to the mail header
    Msg.Fields.Item("urn:schemas:mailheader:X-Received-From-Address").Value = strIP;
    Msg.Fields.Item("urn:schemas:mailheader:X-Envelope-From").Value = GetSender(strSender);
    Msg.Fields.Item("urn:schemas:mailheader:X-Envelope-To").Value = strRecipientHdrs;
    
    // update the mail header and save changes
    Msg.Fields.Update();
    Msg.Datasource.Save();

    // continue execution with the next sink
    Status = cdoRunNextSink;
  }
</SCRIPT> 